var gulp = require('gulp'),
    plugins = require('gulp-load-plugins')(),
    path = require('path'),
    nodemon = require('gulp-nodemon'),
    config = require(path.join(__dirname, 'config', 'gulp.config')),
    paths = config['paths'];

plugins['run-sequence'] = require('run-sequence');
plugins['browser-sync'] = require('browser-sync').create();
plugins['browserify.helper'] = require(path.join(__dirname, 'tasks', 'helpers', 'browserify.js'));

plugins.path = path;

// JavaScript
gulp.task('js',
  require(path.join(__dirname, 'tasks', 'javascript'))(gulp, plugins, paths)
);
gulp.task('jslint',
  require(path.join(__dirname, 'tasks', 'javascript-lint'))(gulp, plugins, paths)
);

// CSS
gulp.task('styl',
  require(path.join(__dirname, 'tasks', 'stylus'))(gulp, plugins, paths)
);
gulp.task('styl-site',
  require(path.join(__dirname, 'tasks', 'stylus-site'))(gulp, plugins, paths)
);
gulp.task('styllint',
  require(path.join(__dirname, 'tasks', 'stylus-lint'))(gulp, plugins, paths)
);
gulp.task('css',
  require(path.join(__dirname, 'tasks', 'css'))(gulp, plugins, paths)
);
gulp.task('csslint',
  require(path.join(__dirname, 'tasks', 'css-lint'))(gulp, plugins, paths)
);

// Images
gulp.task('imagemin',
  require(path.join(__dirname, 'tasks', 'image'))(gulp, plugins, paths)
);

// Fonts
gulp.task('fonts',
  require(path.join(__dirname, 'tasks', 'fonts'))(gulp, plugins, paths)
);

// Lint All
gulp.task('lint',
  ['jslint', 'styllint', 'csslint']
);

// Build All
gulp.task('build', () => {
  return plugins['run-sequence'](
    'clean',
    ['js', 'styl', 'styl-site', 'css', 'imagemin', 'fonts'],
    ['webpack']
  );
});

// Clean dist and build folders
gulp.task('clean',
  require(path.join(__dirname, 'tasks', 'clean'))(gulp, plugins, paths)
);

// Bundle
gulp.task('webpack', plugins['shell'].task(['webpack'], {
  cwd: path.join(__dirname, 'config')
}));
gulp.task('browserify',
  require(path.join(__dirname, 'tasks', 'browserify'))(gulp, plugins, paths)
);

// Livereload
gulp.task('jekyll',
  require(path.join(__dirname, 'tasks', 'jekyll'))(gulp, plugins, paths)
);
gulp.task('browser-sync', ['jekyll'],
  require(path.join(__dirname, 'tasks', 'browser-sync'))(gulp, plugins, paths)
);

// Watchfile
gulp.task('watch',
  require(path.join(__dirname, 'tasks', 'watch'))(gulp, plugins, paths)
);

// Default Task
gulp.task('default', [ 'build', 'browser-sync', 'watch' ])
