const fs = require('fs');
const poststylus = require('poststylus');


module.exports = (gulp, plugins, paths) => {
  var srcPath = plugins['path'].join(paths.src, paths.stylus);
  var destPath = [
    paths.dist,
    plugins['path'].join(paths.site, 'resources')
  ];


  /**
   * Return the first parent file which is not a partial and has the same
   * basename as a folder in a common parent folder
   *
   * @param input Input file received from gulp-rename
   */
  var getParentFile = (input) => {
    var dir = input.dirname.split(plugins['path'].sep);
    var parent;
    var currentDepth = 1;

    do {
      let basedir = dir[ dir.length - currentDepth ];
      parent = plugins['path'].resolve(paths.src, 'css', basedir + input.extname);
      currentDepth += 1;
    } while(!fs.existsSync(parent) && dir[dir.length - currentDepth]);

    return parent;
  }


  /**
   * Handle .styl partial changes by recompiling the parent asset whenever
   * the children are modified.
   *
   * @param input Input file received from gulp-rename
   */
  var partial = ( input ) => {
    if (input.basename[0] == '_') {
      var parent = getParentFile(input);
      delete plugins['cached'].caches['stylus'][parent];
    }

    return input
  }


  return () => {
    return gulp.src(srcPath)
      .pipe(plugins['cached']('stylus'))
      .pipe(plugins['rename'](partial))
      .pipe(plugins['ignore']((file) => /\_.+\.styl$/.test(file.relative) || /index\.styl$/.test(file.relative)))
      .pipe(plugins['stringReplace'](/\@import ([\'\"])\~/g, (replacement, capture) => {
        return '@import ' + capture
      }, {
        logs: {
          enabled: false
        }
      }))
      .pipe(plugins['debug']())
      .pipe(plugins['sourcemaps'].init())
      .pipe(plugins['plumber']())
      .pipe(plugins['stylus']({
        'resolve url': true,
        'include css': true,
        'include': 'node_modules',
        'modulesDirectories': ['node_modules'],
        'url': 'embedurl',
        'use': [
          poststylus([
            'autoprefixer',
            'rucksack-css'
          ])
        ]
      }))
      .pipe(plugins['sourcemaps'].write(plugins['path'].join('.')))
      .pipe(gulp.dest(paths.build))
      .pipe(plugins['filter']('**/*.css'))
      .pipe(plugins['browser-sync'].stream())
      .pipe(plugins['ignore'].exclude([ '**/*.map' ]))
      .pipe(plugins['cssmin']())
      .pipe(gulp.dest(destPath[0]))
      .pipe(plugins['rename'](function(file) {
        file.dirname = plugins['path'].join('css', 'inkline');
        console.log(file)
      }))
      .pipe(gulp.dest(destPath[1]));
  }
}
